"""J.A.R.V.I.S. Starboard Cog."""
from dis_snek import InteractionContext, Permissions, Scale, Snake
from dis_snek.models.discord.channel import GuildText
from dis_snek.models.discord.components import ActionRow, Select, SelectOption
from dis_snek.models.discord.message import Message
from dis_snek.models.snek.application_commands import (
    CommandTypes,
    OptionTypes,
    context_menu,
    slash_command,
    slash_option,
)
from dis_snek.models.snek.command import check

from jarvis.db.models import Star, Starboard
from jarvis.utils import build_embed, find
from jarvis.utils.permissions import admin_or_permissions

supported_images = [
    "image/png",
    "image/gif",
    "image/jpeg",
    "image/webp",
    "image/svg",
]


class StarboardCog(Scale):
    """J.A.R.V.I.S. Starboard Cog."""

    def __init__(self, bot: Snake):
        self.bot = bot

    @slash_command(name="starboard", sub_cmd_name="list", sub_cmd_description="List all starboards")
    @check(admin_or_permissions(Permissions.MANAGE_GUILD))
    async def _list(self, ctx: InteractionContext) -> None:
        starboards = Starboard.objects(guild=ctx.guild.id)
        if starboards != []:
            message = "Available Starboards:\n"
            for s in starboards:
                message += f"<#{s.channel}>\n"
            await ctx.send(message)
        else:
            await ctx.send("No Starboards available.")

    @slash_command(
        name="starboard", sub_cmd_name="create", sub_cmd_description="Create a starboard"
    )
    @slash_option(
        name="channel",
        description="Starboard channel",
        opt_type=OptionTypes.CHANNEL,
        required=True,
    )
    @check(admin_or_permissions(Permissions.MANAGE_GUILD))
    async def _create(self, ctx: InteractionContext, channel: GuildText) -> None:
        if channel not in ctx.guild.channels:
            await ctx.send(
                "Channel not in guild. Choose an existing channel.",
                ephemeral=True,
            )
            return
        if not isinstance(channel, GuildText):
            await ctx.send("Channel must be a GuildText", ephemeral=True)
            return

        exists = Starboard.objects(channel=channel.id, guild=ctx.guild.id).first()
        if exists:
            await ctx.send(f"Starboard already exists at {channel.mention}.", ephemeral=True)
            return

        count = Starboard.objects(guild=ctx.guild.id).count()
        if count >= 25:
            await ctx.send("25 starboard limit reached", ephemeral=True)
            return

        _ = Starboard(
            guild=ctx.guild.id,
            channel=channel.id,
            admin=ctx.author.id,
        ).save()
        await ctx.send(f"Starboard created. Check it out at {channel.mention}.")

    @slash_command(
        name="starboard", sub_cmd_name="delete", sub_cmd_description="Delete a starboard"
    )
    @slash_option(
        name="channel",
        description="Starboard channel",
        opt_type=OptionTypes.CHANNEL,
        required=True,
    )
    @check(admin_or_permissions(Permissions.MANAGE_GUILD))
    async def _delete(self, ctx: InteractionContext, channel: GuildText) -> None:
        deleted = Starboard.objects(channel=channel.id, guild=ctx.guild.id).delete()
        if deleted:
            _ = Star.objects(starboard=channel.id).delete()
            await ctx.send(f"Starboard deleted from {channel.mention}.", ephemeral=True)
        else:
            await ctx.send(f"Starboard not found in {channel.mention}.", ephemeral=True)

    @context_menu(name="Star Message", context_type=CommandTypes.MESSAGE)
    async def _star_message(self, ctx: InteractionContext) -> None:
        await self._star_add.invoke(ctx, ctx.target_message)

    @slash_command(name="star", sub_cmd_name="add", description="Star a message")
    @slash_option(
        name="message", description="Message to star", opt_type=OptionTypes.STRING, required=True
    )
    @slash_option(
        name="channel",
        description="Channel that has the message, not required if used in same channel",
        opt_type=OptionTypes.CHANNEL,
        required=False,
    )
    @check(admin_or_permissions(Permissions.MANAGE_GUILD))
    async def _star_add(
        self,
        ctx: InteractionContext,
        message: str,
        channel: GuildText = None,
    ) -> None:
        if not channel:
            channel = ctx.channel
        starboards = Starboard.objects(guild=ctx.guild.id)
        if not starboards:
            await ctx.send("No starboards exist.", ephemeral=True)
            return

        await ctx.defer()

        if not isinstance(message, Message):
            if message.startswith("https://"):
                message = message.split("/")[-1]
            message = await channel.get_message(int(message))

            if not message:
                await ctx.send("Message not found", ephemeral=True)
                return

        channel_list = []
        for starboard in starboards:
            channel_list.append(find(lambda x: x.id == starboard.channel, ctx.guild.channels))

        select_channels = [
            SelectOption(label=x.name, value=str(idx)) for idx, x in enumerate(channel_list)
        ]

        select = Select(
            options=select_channels,
            min_values=1,
            max_values=1,
        )

        components = [ActionRow(select)]

        msg = await ctx.send(content="Choose a starboard", components=components)

        com_ctx = await self.bot.wait_for_component(
            messages=msg,
            components=components,
            check=lambda x: x.author.id == ctx.author.id,
        )

        starboard = channel_list[int(com_ctx.selected_options[0])]

        exists = Star.objects(
            message=message.id,
            channel=message.channel.id,
            guild=message.guild.id,
            starboard=starboard.id,
        ).first()

        if exists:
            await ctx.send(
                f"Message already sent to Starboard {starboard.mention}",
                ephemeral=True,
            )
            return

        count = Star.objects(guild=message.guild.id, starboard=starboard.id).count()
        content = message.content

        attachments = message.attachments
        image_url = None
        if attachments:
            for attachment in attachments:
                if attachment.content_type in supported_images:
                    image_url = attachment.url
                    break
        if not content and image_url:
            content = "\u200b"

        embed = build_embed(
            title=f"[#{count}] Click Here to view context",
            description=content,
            fields=[],
            url=message.jump_url,
            timestamp=message.created_at,
        )
        embed.set_author(
            name=message.author.display_name,
            url=message.jump_url,
            icon_url=message.author.display_avatar.url,
        )
        embed.set_footer(text=message.guild.name + " | " + message.channel.name)
        if image_url:
            embed.set_image(url=image_url)

        star = await starboard.send(embed=embed)

        _ = Star(
            index=count,
            message=message.id,
            channel=message.channel.id,
            guild=message.guild.id,
            starboard=starboard.id,
            admin=ctx.author.id,
            star=star.id,
            active=True,
        ).save()

        components[0]["components"][0]["disabled"] = True

        await com_ctx.edit_origin(
            content=f"Message saved to Starboard.\nSee it in {starboard.mention}",
            components=components,
        )

    @slash_command(name="star", sub_cmd_name="delete", description="Delete a starred message")
    @slash_option(
        name="message", description="Star to delete", opt_type=OptionTypes.INTEGER, required=True
    )
    @slash_option(
        name="starboard",
        description="Starboard to delete star from",
        opt_type=OptionTypes.CHANNEL,
        required=True,
    )
    @check(admin_or_permissions(Permissions.MANAGE_GUILD))
    async def _star_delete(
        self,
        ctx: InteractionContext,
        id: int,
        starboard: GuildText,
    ) -> None:
        if not isinstance(starboard, GuildText):
            await ctx.send("Channel must be a GuildText channel", ephemeral=True)
            return
        exists = Starboard.objects(channel=starboard.id, guild=ctx.guild.id).first()
        if not exists:
            await ctx.send(
                f"Starboard does not exist in {starboard.mention}. Please create it first",
                ephemeral=True,
            )
            return

        star = Star.objects(
            starboard=starboard.id,
            index=id,
            guild=ctx.guild.id,
            active=True,
        ).first()
        if not star:
            await ctx.send(f"No star exists with id {id}", ephemeral=True)
            return

        message = await starboard.fetch_message(star.star)
        if message:
            await message.delete()

        star.active = False
        star.save()

        await ctx.send(f"Star {id} deleted")


def setup(bot: Snake) -> None:
    """Add StarboardCog to J.A.R.V.I.S."""
    StarboardCog(bot)
